@@ -41,10 +41,10 @@ def _b64url_decode(s: str) -> bytes:
 def _decode_jwt_payload(jwt: str) -> dict:
     """
     Decode JWT payload without verification (for exp checks only).
-    
+
     Args:
         jwt: JWT token string
-        
+
     Returns:
         Decoded payload dict
     """
@@ -59,6 +59,259 @@ def _decode_jwt_payload(jwt: str) -> dict:
         raise ValueError(f"Failed to decode JWT payload: {e}")
 
 
+def _decode_jwt_header(jwt: str) -> dict:
+    """
+    Decode JWT header without verification.
+
+    Args:
+        jwt: JWT token string
+
+    Returns:
+        Decoded header dict (contains alg, kid, typ)
+    """
+    try:
+        parts = jwt.split('.')
+        if len(parts) != 3:
+            raise ValueError("Invalid JWT format")
+        header_b64url = parts[0]
+        header_bytes = _b64url_decode(header_b64url)
+        return json.loads(header_bytes.decode('utf-8'))
+    except Exception as e:
+        raise ValueError(f"Failed to decode JWT header: {e}")
+
+
+# ============================================
+# JWKS and Local JWT Verification
+# ============================================
+
+# JWKS cache
+_cached_jwks: Optional[dict] = None
+_jwks_cache_time: float = 0
+_JWKS_CACHE_TTL = 3600  # 1 hour in seconds
+
+
+async def _fetch_jwks() -> dict:
+    """
+    Fetch JWKS from gateway.
+
+    Returns:
+        JWKS dict with 'keys' array
+    """
+    global _cached_jwks, _jwks_cache_time
+
+    # Check cache first
+    if _cached_jwks and time.time() - _jwks_cache_time < _JWKS_CACHE_TTL:
+        return _cached_jwks
+
+    from ..config.endpoints import get_gateway_url
+    gateway_url = get_gateway_url()
+
+    try:
+        async with httpx.AsyncClient(timeout=10.0) as client:
+            response = await client.get(f"{gateway_url}/jwt/jwks")
+            response.raise_for_status()
+
+            # Response is base64 encoded
+            result = _b64_decode(response.text)
+
+            # Handle wrapped response or direct JWKS
+            if isinstance(result, dict):
+                if result.get("data", {}).get("keys"):
+                    jwks = result["data"]
+                elif result.get("keys"):
+                    jwks = result
+                else:
+                    raise ValueError("Invalid JWKS response format")
+            else:
+                raise ValueError("Invalid JWKS response format")
+
+            _cached_jwks = jwks
+            _jwks_cache_time = time.time()
+            return jwks
+
+    except Exception as e:
+        # If fetch fails but we have cached JWKS, use it (stale is better than nothing)
+        if _cached_jwks:
+            return _cached_jwks
+        raise RuntimeError(f"Failed to fetch JWKS: {e}")
+
+
+def _jwk_to_rsa_public_key(jwk: dict):
+    """
+    Convert JWK to RSA public key for verification.
+
+    Args:
+        jwk: JWK dict with n, e components
+
+    Returns:
+        RSA public key object
+    """
+    from cryptography.hazmat.primitives.asymmetric import rsa
+    from cryptography.hazmat.backends import default_backend
+
+    # Decode n and e from base64url
+    n_bytes = _b64url_decode(jwk["n"])
+    e_bytes = _b64url_decode(jwk["e"])
+
+    # Convert to integers
+    n = int.from_bytes(n_bytes, byteorder='big')
+    e = int.from_bytes(e_bytes, byteorder='big')
+
+    # Create RSA public numbers and derive public key
+    public_numbers = rsa.RSAPublicNumbers(e, n)
+    return public_numbers.public_key(default_backend())
+
+
+async def verify_jwt_signature(jwt_token: str) -> dict:
+    """
+    Verify JWT signature locally using JWKS.
+
+    Args:
+        jwt_token: The JWT token to verify
+
+    Returns:
+        The decoded payload if valid
+
+    Raises:
+        RuntimeError: If verification fails
+    """
+    global _cached_jwks
+
+    from cryptography.hazmat.primitives import hashes
+    from cryptography.hazmat.primitives.asymmetric import padding
+    from cryptography.exceptions import InvalidSignature
+
+    parts = jwt_token.split('.')
+    if len(parts) != 3:
+        raise RuntimeError("Invalid JWT format")
+
+    header_b64, payload_b64, signature_b64 = parts
+
+    # Decode header to get kid
+    header = _decode_jwt_header(jwt_token)
+
+    # Verify algorithm
+    if header.get("alg") != "RS256":
+        raise RuntimeError(f"Unsupported JWT algorithm: {header.get('alg')}")
+
+    # Fetch JWKS and find the key
+    jwks = await _fetch_jwks()
+    kid = header.get("kid")
+
+    key = None
+    if kid:
+        for k in jwks.get("keys", []):
+            if k.get("kid") == kid:
+                key = k
+                break
+    else:
+        # Use first key if no kid
+        keys = jwks.get("keys", [])
+        if keys:
+            key = keys[0]
+
+    if not key:
+        # Key not found - refresh JWKS and try again
+        _cached_jwks = None
+        jwks = await _fetch_jwks()
+
+        if kid:
+            for k in jwks.get("keys", []):
+                if k.get("kid") == kid:
+                    key = k
+                    break
+        else:
+            keys = jwks.get("keys", [])
+            if keys:
+                key = keys[0]
+
+        if not key:
+            raise RuntimeError(f"JWT signing key not found: {kid}")
+
+    # Import the public key
+    public_key = _jwk_to_rsa_public_key(key)
+
+    # Verify signature
+    signature_input = f"{header_b64}.{payload_b64}".encode('utf-8')
+    signature = _b64url_decode(signature_b64)
+
+    try:
+        public_key.verify(
+            signature,
+            signature_input,
+            padding.PKCS1v15(),
+            hashes.SHA256()
+        )
+    except InvalidSignature:
+        raise RuntimeError("JWT signature verification failed")
+
+    # Decode and return payload
+    return _decode_jwt_payload(jwt_token)
+
+
+async def verify_jwt_locally(jwt_token: str) -> dict:
+    """
+    Verify JWT locally: signature + claims.
+
+    Performs full local verification:
+    1. Signature verification using JWKS
+    2. Expiry check (exp claim)
+    3. Not-before check (nbf claim if present)
+    4. Issued-at sanity check (iat claim)
+
+    Args:
+        jwt_token: The JWT token to verify
+
+    Returns:
+        The decoded payload if valid
+
+    Raises:
+        RuntimeError: If verification fails
+    """
+    # Verify signature
+    payload = await verify_jwt_signature(jwt_token)
+
+    now = int(time.time())
+
+    # Check expiry
+    if payload.get("exp") and payload["exp"] < now:
+        raise RuntimeError("JWT has expired")
+
+    # Check not-before
+    if payload.get("nbf") and payload["nbf"] > now:
+        raise RuntimeError("JWT not yet valid")
+
+    # Sanity check issued-at (not more than 24 hours in the future)
+    if payload.get("iat") and payload["iat"] > now + 86400:
+        raise RuntimeError("JWT issued-at is in the future")
+
+    return payload
+
+
+def is_jwt_valid(jwt_token: str) -> bool:
+    """
+    Quick local JWT validation (expiry only, no signature verification).
+    Use this for fast checks before making API calls.
+
+    Args:
+        jwt_token: The JWT token to check
+
+    Returns:
+        True if token appears valid (not expired), False otherwise
+    """
+    try:
+        payload = _decode_jwt_payload(jwt_token)
+        now = int(time.time())
+
+        # Check if expired (with 60 second buffer)
+        if payload.get("exp") and payload["exp"] < now - 60:
+            return False
+
+        return True
+    except Exception:
+        return False
+
+
 async def _get_service_jwt(psk_token: str, base_url: str) -> str:
     """
     Get service JWT by calling gateway /jwt/mint with PSK.
