@@ -54,10 +54,14 @@ class OpenNamespace:
         Execute raw SQL query.
 
         Use with caution - this bypasses most safety checks.
+        
+        Note: The backend only accepts raw SQL (no parameterized queries).
+        If params are provided, they are ignored. Format your SQL string
+        with proper escaping before calling this method.
 
         Args:
-            sql: SQL query string
-            params: Optional parameter dictionary
+            sql: SQL query string (must be fully formatted, no $1, $2 placeholders)
+            params: Optional parameter dictionary (ignored - backend doesn't support params)
 
         Returns:
             Query result
@@ -66,8 +70,7 @@ class OpenNamespace:
         # Backend expects sql_b64 (base64-encoded SQL)
         sql_b64 = base64.b64encode(sql.encode('utf-8')).decode('utf-8')
         body = {"sql": sql_b64}
-        if params:
-            body["params"] = params
+        # Note: params are ignored - backend only accepts raw SQL
 
         return await self._client._request(
             endpoint="/api/scribe/open/execute",
