@@ -59,15 +59,17 @@ from typing import Optional, Any, Dict, List
 _VALIDATED = False
 _TOKEN: Optional[str] = None
 _BASE_URL: Optional[str] = None
+_GATEWAY_URL: Optional[str] = None
 _VALIDATION_ERROR: Optional[str] = None
 
 
 # === MODULE-LEVEL INITIALIZATION ===
 from .helpers.auth import _resolve_token
-from .config.endpoints import BASE_URL
+from .config.endpoints import BASE_URL, GATEWAY_URL
 
 _TOKEN = _resolve_token()
 _BASE_URL = BASE_URL
+_GATEWAY_URL = GATEWAY_URL
 
 # Initialize cache encryption
 from .helpers.cache import dominus_cache
@@ -365,10 +367,11 @@ class Dominus:
         endpoint: str,
         method: str = "POST",
         body: Optional[Dict[str, Any]] = None,
-        user_token: Optional[str] = None
+        user_token: Optional[str] = None,
+        use_gateway: bool = False
     ) -> Dict[str, Any]:
         """
-        Make an HTTP request to the orchestrator.
+        Make an HTTP request to the orchestrator or gateway.
 
         If user_token is provided, uses that JWT directly (for user-authenticated requests).
         Otherwise, uses the service token flow (PSK -> JWT).
@@ -378,6 +381,7 @@ class Dominus:
             method: HTTP method (GET, POST, PUT, DELETE)
             body: Request body (will be base64-encoded)
             user_token: Optional user JWT for user-authenticated requests
+            use_gateway: If True, route through gateway (/api/* -> /svc/*)
 
         Returns:
             Response data dict
@@ -391,6 +395,10 @@ class Dominus:
         def _b64_decode(s: str) -> dict:
             return json.loads(base64.b64decode(s.encode('utf-8')).decode('utf-8'))
 
+        # Transform endpoint for gateway routing
+        if use_gateway and endpoint.startswith("/api/"):
+            endpoint = "/svc/" + endpoint[5:]  # /api/llm/complete -> /svc/llm/complete
+
         # Determine which JWT to use
         if user_token:
             # Use the provided user JWT directly (no validation needed - orchestrator validates it)
@@ -426,8 +434,11 @@ class Dominus:
         # Prepare body
         body_b64 = _b64_encode(body or {})
 
+        # Select base URL
+        base_url = _GATEWAY_URL if use_gateway else _BASE_URL
+
         # Make request
-        async with httpx.AsyncClient(base_url=_BASE_URL, headers=headers, timeout=30.0) as client:
+        async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=30.0) as client:
             if method == "GET":
                 response = await client.get(endpoint)
             elif method == "DELETE":
@@ -454,16 +465,18 @@ class Dominus:
         endpoint: str,
         body: Optional[Dict[str, Any]] = None,
         on_chunk: Optional[Any] = None,
-        timeout: float = 300.0
+        timeout: float = 300.0,
+        use_gateway: bool = False
     ):
         """
-        Make an SSE streaming request to the orchestrator.
+        Make an SSE streaming request to the orchestrator or gateway.
 
         Args:
             endpoint: API endpoint path (e.g., "/api/agent/stream")
             body: Request body (will be base64-encoded)
             on_chunk: Optional callback for each chunk
             timeout: Request timeout in seconds (default: 300s for long operations)
+            use_gateway: If True, route through gateway (/api/* -> /svc/*)
 
         Yields:
             Parsed JSON chunks from SSE stream
@@ -474,6 +487,10 @@ class Dominus:
         def _b64_encode(d: dict) -> str:
             return base64.b64encode(json.dumps(d).encode()).decode()
 
+        # Transform endpoint for gateway routing
+        if use_gateway and endpoint.startswith("/api/"):
+            endpoint = "/svc/" + endpoint[5:]
+
         # Validate and get JWT
         if _VALIDATION_ERROR:
             raise RuntimeError(_VALIDATION_ERROR)
@@ -499,7 +516,8 @@ class Dominus:
 
         from .helpers.sse import stream_sse
 
-        async with httpx.AsyncClient(base_url=_BASE_URL, timeout=timeout) as client:
+        base_url = _GATEWAY_URL if use_gateway else _BASE_URL
+        async with httpx.AsyncClient(base_url=base_url, timeout=timeout) as client:
             async with client.stream("POST", endpoint, content=body_b64, headers=headers) as response:
                 response.raise_for_status()
                 async for chunk in stream_sse(response, on_chunk):
@@ -512,7 +530,8 @@ class Dominus:
         filename: str,
         content_type: str = "application/octet-stream",
         additional_fields: Optional[Dict[str, str]] = None,
-        timeout: float = 60.0
+        timeout: float = 60.0,
+        use_gateway: bool = False
     ) -> Dict[str, Any]:
         """
         Upload binary file via multipart/form-data (no base64 encoding).
@@ -526,6 +545,7 @@ class Dominus:
             content_type: MIME type (default: application/octet-stream)
             additional_fields: Optional additional form fields
             timeout: Request timeout in seconds
+            use_gateway: If True, route through gateway (/api/* -> /svc/*)
 
         Returns:
             Response data dict
@@ -535,6 +555,10 @@ class Dominus:
         def _b64_decode(s: str) -> dict:
             return json.loads(base64.b64decode(s.encode('utf-8')).decode('utf-8'))
 
+        # Transform endpoint for gateway routing
+        if use_gateway and endpoint.startswith("/api/"):
+            endpoint = "/svc/" + endpoint[5:]
+
         if _VALIDATION_ERROR:
             raise RuntimeError(_VALIDATION_ERROR)
 
@@ -557,7 +581,8 @@ class Dominus:
         files = {"file": (filename, file_bytes, content_type)}
         data = additional_fields or {}
 
-        async with httpx.AsyncClient(base_url=_BASE_URL, headers=headers, timeout=timeout) as client:
+        base_url = _GATEWAY_URL if use_gateway else _BASE_URL
+        async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=timeout) as client:
             response = await client.post(endpoint, files=files, data=data)
 
         response.raise_for_status()
@@ -578,7 +603,8 @@ class Dominus:
         self,
         endpoint: str,
         body: Optional[Dict[str, Any]] = None,
-        timeout: float = 60.0
+        timeout: float = 60.0,
+        use_gateway: bool = False
     ) -> bytes:
         """
         Download binary content (no base64 encoding in response).
@@ -589,6 +615,7 @@ class Dominus:
             endpoint: API endpoint path (e.g., "/api/agent/tts")
             body: Request body (will be base64-encoded for auth)
             timeout: Request timeout in seconds
+            use_gateway: If True, route through gateway (/api/* -> /svc/*)
 
         Returns:
             Raw binary bytes
@@ -598,6 +625,10 @@ class Dominus:
         def _b64_encode(d: dict) -> str:
             return base64.b64encode(json.dumps(d).encode()).decode()
 
+        # Transform endpoint for gateway routing
+        if use_gateway and endpoint.startswith("/api/"):
+            endpoint = "/svc/" + endpoint[5:]
+
         if _VALIDATION_ERROR:
             raise RuntimeError(_VALIDATION_ERROR)
 
@@ -620,7 +651,8 @@ class Dominus:
 
         body_b64 = _b64_encode(body or {})
 
-        async with httpx.AsyncClient(base_url=_BASE_URL, headers=headers, timeout=timeout) as client:
+        base_url = _GATEWAY_URL if use_gateway else _BASE_URL
+        async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=timeout) as client:
             response = await client.post(endpoint, content=body_b64)
 
         response.raise_for_status()
